// Code based on the ESP32 example code TouchInterupt and code from 
// https://iotdesignpro.com/projects/creating-nodemcu-mesh-networking-using-esp12-and-arduino-ide

// Code for Node1:
#include "painlessMesh.h"   // include painlessMesh library


// WiFi Credentials
#define   MESH_PREFIX     "Post-collapseNetwork"
#define   MESH_PASSWORD   "TechnoLogic"
#define   MESH_PORT       5555

int threshold = 40;    // setting threshold for detecting touch on pin
bool touch1detected = false;   
bool touch2detected = false;
bool touch3detected = false;
bool touch4detected = false;
bool touch5detected = false;

//Pin Declaration
#define touchPin1 T5  // declaring touch pins
#define touchPin2 T2 //Touchpin numberbering can be found at https://gitlab.fabcloud.org/barcelonaworkshops/barduino-2.0/
#define touchPin3 T7 
#define touchPin4 T3
#define touchPin5 T0

Scheduler userScheduler; // calling on the scheduler to control your personal task. This replaces 'delay' function which messes up painlessMesh 
painlessMesh  mesh;  // calling on painlessMesh

void sendMessage() ;  // Prototype so PlatformIO doesn't complain
Task taskSendMessage( TASK_SECOND * 1 , TASK_FOREVER, &sendMessage );  // this command replaces the 'delay' function
void sendMessage() 
{
// Reading Status of touchPin
  if (digitalRead(touchPin1) == HIGH)
    touch1detected = !touch1detected;  // creates container to put status of pin in.
  if (digitalRead(touchPin2) == HIGH)
    touch2detected = !touch2detected;
  if (digitalRead(touchPin3) == HIGH)
    touch3detected = !touch3detected;
  if (digitalRead(touchPin4) == HIGH)
    touch4detected = !touch4detected;
  if (digitalRead(touchPin5) == HIGH)
    touch5detected = !touch5detected;
    
// Serializing in JSON Format 
  DynamicJsonDocument doc(1024);
  doc["Relay1"] = touch1detected; // container created above is made into a doc named Relay1 file.
  doc["Relay2"] = touch2detected; // container created above is made into a doc named Relay2 file.
  doc["Relay3"] = touch3detected; 
  doc["Relay4"] = touch4detected; 
  doc["Relay5"] = touch4detected;
  String msg ;
  serializeJson(doc, msg);   // 
    }


// Needed for painless library
void receivedCallback( uint32_t from, String &msg ) {
  Serial.printf("startHere: Received from %u msg=%s\n", from, msg.c_str());
}

void newConnectionCallback(uint32_t nodeId) {
  Serial.printf("--> startHere: New Connection, nodeId = %u\n", nodeId);
}
void changedConnectionCallback() {
  Serial.printf("Changed connections\n");
}
void nodeTimeAdjustedCallback(int32_t offset) {
  Serial.printf("Adjusted time %u. Offset = %d\n", mesh.getNodeTime(), offset);
}

void gotTouch1(){
 touch1detected = true;
}
void gotTouch2(){
 touch2detected = true;
}
void gotTouch3(){
 touch3detected = true;
}
void gotTouch4(){
 touch4detected = true;
}
void gotTouch5(){
 touch4detected = true;
}


void setup() {
  Serial.begin(115200);
  touchAttachInterrupt(T5, gotTouch1, threshold);  // checks if touchpin is interupted
  touchAttachInterrupt(T2, gotTouch2, threshold);
  touchAttachInterrupt(T7, gotTouch3, threshold);
  touchAttachInterrupt(T3, gotTouch4, threshold);
  touchAttachInterrupt(T0, gotTouch4, threshold);
  mesh.setDebugMsgTypes( ERROR | STARTUP );  // set before init() so that you can see startup messages
  mesh.init( MESH_PREFIX, MESH_PASSWORD, &userScheduler, MESH_PORT );
  mesh.onReceive(&receivedCallback);
  mesh.onNewConnection(&newConnectionCallback);
  mesh.onChangedConnections(&changedConnectionCallback);
  mesh.onNodeTimeAdjusted(&nodeTimeAdjustedCallback);
  userScheduler.addTask( taskSendMessage );
  taskSendMessage.enable();
}
void loop() {
    mesh.update();
  
  if(touch1detected){
    touch1detected = false;
    Serial.println("Touch 1 detected");
    
    sendMessage();   // Broadcasting message
    String msg = ".";
    mesh.sendBroadcast( msg );
}
  
  if(touch2detected){
    touch2detected = false;
    Serial.println("Touch 2 detected");

  sendMessage();   
    String msg = "-";
    mesh.sendBroadcast( msg );  
  }

 if(touch3detected){
    touch3detected = false;
    Serial.println("Touch 3 detected");

  sendMessage();   
    String msg = "/";
    mesh.sendBroadcast( msg );
 }    

 if(touch4detected){
    touch4detected = false;
    Serial.println("Touch 4 detected");

  sendMessage();   
    String msg = " ";
    mesh.sendBroadcast( msg );
 }    
 if(touch5detected){
    touch5detected = false;
    Serial.println("Touch 5 detected");

  sendMessage();   
    String msg = " ";
    mesh.sendBroadcast( msg );
 }    
}
